@@ -21,6 +21,7 @@
 #' \item \code{\linkS4class{DeponsDyn}} population dynamics data, from "Statistics.XXX.csv" files
 #' \item \code{\linkS4class{DeponsBlockdyn}} data from "PorpoisePerBlock.XXX.csv" files
 #' \item \code{\linkS4class{DeponsShips}} data from "ships.json" files or from AIS data
+#' \item \code{\linkS4class{DeponsRaster}} stores raster landscape files used in DEPONS
 #' }
 #' Here the \code{DeponsDyn} data include both changes in population size and energetics
 #' through time for the entire landscape, whereas \code{DeponsBlockdyn} data include
@@ -343,7 +344,6 @@ tick.to.time <- function(tick, timestep=30, origin="2010-01-01", tz = "UTC", ...
 
 
 
-
 #'Convert date to tick number
 #'
 #'@description
@@ -432,7 +432,7 @@ time.to.tick <- function (time, timestep = 30, origin = "2010-01-01", tz = "UTC"
     breaks <- 0
     while (breaks == 0) { # then proceed through remaining months
       # reset format to roll over added months
-      cur.or <- as.POSIXlt(as.POSIXct(cur.or, tz = "UTC"), tz = "UTC")
+      cur.or <- as.POSIXlt(as.POSIXct(cur.or, tz = tz), tz = tz)
       if (cur.or > target) stop (paste0("tick accumulation overshot target date. Failed at ", time[instance]))
       cur.diff <- as.numeric(difftime(target, cur.or, units = "secs"))
       if (cur.or$year == target$year && cur.or$mon == target$mon) { # if reached same month and same year, add remainder secs to count
@@ -448,6 +448,8 @@ time.to.tick <- function (time, timestep = 30, origin = "2010-01-01", tz = "UTC"
     ticks <- floor(cur.count / (60 * timestep))
     return (ticks)
   }
+
+  time <- as.character(time)
   time.converted <- time
   for (instance in 1:length(time)) {
     test <- try(as.POSIXlt(time[instance], tz = tz))
